@@ -1,4 +1,5 @@
 from stdnet.exceptions import *
+from structures import Structure
 
 novalue = object()
 
@@ -22,20 +23,25 @@ class NoPickle(object):
 nopickle = NoPickle()
 
 
-class Pipeline(object):
+class Keys(object):
+    
+    def __init__(self,id,timeout,pipeline):
+        self.timeout = timeout
+        self.value = None
+        pipeline[id] = self
+        
+    def add(self, value):
+        self.value = value
     
-    def __init__(self, pipe, method):
-        self.pipe   = pipe
-        self.method = method
 
-
-class BaseBackend(object):
+class BackendDataServer(object):
     '''Generic interface for a backend database:
     
     * *name* name of database, such as **redis**, **couchdb**, etc..
     * *params* dictionary of configuration parameters
     * *pickler* calss for serializing and unserializing data. It must implement the *loads* and *dumps* methods.
     '''
+    structure_module = None
     def __init__(self, name, params, pickler = None):
         self.__name = name
         timeout = params.get('timeout', 0)
@@ -46,7 +52,6 @@ class BaseBackend(object):
         self.default_timeout = timeout
         self._cachepipe = {}
         self._keys      = {}
-        self.fields     = []
         self.params     = params
         self.pickler    = pickler or default_pickler
 
@@ -63,102 +68,107 @@ class BaseBackend(object):
     def createdb(self, name):
         pass
     
+    def delete(self, *key):
+        "Delete one or more keys specified by ``keys``"
+        raise NotImplementedError
+    
     def get_object(self, meta, name, value):
-        '''Retrive an object from the database. If object is not available, it raise
-and :ref:`ObjectNotFund <utility-exceptions>` exception.
+        '''Retrive an object from the database. If object is not available, it raises
+an :class:`stdnet.exceptions.ObjectNotFund` exception.
 
     * *meta* :ref:`database metaclass <database-metaclass>` or model
     * *name* name of field (must be unique)
     * *value* value of field to search.'''
         if name != 'id':
-            value = self._get(meta.basekey(name,value))
-        if value is None:
+            id = self._get(meta.basekey(name,value))
+        else:
+            id = value
+        if id is None:
             raise ObjectNotFund
-        value = self.hash(meta.basekey()).get(value)
-        if value is None:
+        data = self.hash(meta.basekey()).get(id)
+        if data is None:
             raise ObjectNotFund
-        return value
+        return meta.make(id,data)
     
-    def add_object(self, obj, commit = True):
+    def _get_pipe(self, id, typ, timeout):
+        cache  = self._cachepipe
+        cvalue = cache.get(id,None)
+        if cvalue is None:
+            cvalue = typ(timeout)
+            cache[id] = cvalue
+        return cvalue
+            
+    def add_object(self, obj, data, indexes, commit = True):
         '''Add a model object to the database:
         
         * *obj* instance of :ref:`StdModel <model-model>` to add to database
         * *commit* If True, *obj* is saved to database, otherwise it remains in local cache.
         '''
-        meta   = obj._meta
-        id     = meta.basekey()
-        cache  = self._cachepipe
-        cvalue = cache.get(id,None)
-        if cvalue is None:
-            cvalue = Pipeline({},'hash')
-            cache[id] = cvalue
-        hash = self.hash(id, meta.timeout, cvalue.pipe)
+        meta  = obj._meta
+        timeout = meta.timeout
+        cache = self._cachepipe
+        hash  = meta.table()
         objid = obj.id
-        hash.add(objid, obj)
+        hash.add(objid, data)
         
         # Create indexes if possible
-        for name,field in meta.fields.items():
-            if name is 'id':
-                continue
-            if field.index:
-                value   = field.hash(field.serialize())
-                key     = meta.basekey(field.name,value)
-                if field.unique:
-                    self._keys[key] = objid
-                elif field.ordered:                    
-                    cvalue = cache.get(key,None)
-                    if cvalue is None:
-                        cvalue = Pipeline(set(),'ordered_set')
-                        cache[key] = cvalue
-                    index = self.ordered_set(key,
-                                             timeout  = meta.timeout,
-                                             pipeline = cvalue.pipe,
-                                             pickler  = nopickle)
-                    index.add(objid)
+        for field,value in indexes:
+            key     = meta.basekey(field.name,value)
+            if field.unique:
+                index = self.index_keys(key, timeout)
+            else:
+                if field.ordered:
+                    index = self.ordered_set(key, timeout, pickler = nopickle)
                 else:
-                    cvalue = cache.get(key,None)
-                    if cvalue is None:
-                        cvalue = Pipeline(set(),'unordered_set')
-                        cache[key] = cvalue
-                    index = self.unordered_set(key,
-                                               timeout  = meta.timeout,
-                                               pipeline = cvalue.pipe,
-                                               pickler  = nopickle)
-                    index.add(objid)
-            
-            savefield = getattr(field,'save',None)
-            if savefield:
-                self.fields.append(savefield)
+                    index = self.unordered_set(key, timeout, pickler = nopickle)
+            index.add(objid)
                 
         if commit:
             self.commit()
             
     def commit(self):
-        '''Commit cache objects to backend database'''
-        for id,cvalue in self._cachepipe.iteritems():
-            el = getattr(self,cvalue.method)(id, pipeline = cvalue.pipe)
+        '''Commit cache objects to database.'''
+        cache = self._cachepipe
+        keys = self._keys
+        # flush cache
+        self._cachepipe = {}
+        self._keys = {}
+        # commit
+        for id,pipe in cache.iteritems():
+            el = getattr(self,pipe.method)(id, pipeline = pipe)
             el.save()
-        if self._keys:
-            self._set_keys()
-            self._keys.clear()
-        for save in self.fields:
-            save()
-        self.fields = []            
+        if keys: 
+            self._set_keys(keys)
             
-    def delete_object(self, obj):
-        '''Delete an object from the database'''
-        meta   = obj._meta
-        id     = meta.basekey()
-        hash   = self.hash(id)
-        if not hash.delete(obj.id):
+    def delete_object(self, obj, deleted = None):
+        '''Delete an object from the data server and clean up indices.'''
+        deleted = deleted if deleted is not None else []
+        meta    = obj._meta
+        timeout = meta.timeout
+        hash    = meta.table()
+        bkey    = meta.basekey
+        objid   = obj.id
+        if not hash.delete(objid):
             return 0
-        self.delete_indexes(obj)
+        for field in meta.fields:
+            name = field.name
+            if field.index:
+                key   = bkey(name,field.serialize(getattr(obj,name,None)))
+                if field.unique:
+                    deleted.append(self.delete(key))
+                else:
+                    if field.ordered:
+                        idx = self.ordered_set(key, timeout, pickler = nopickle)
+                    else:
+                        idx = self.unordered_set(key, timeout, pickler = nopickle)
+                    deleted.append(idx.discard(objid))
+            fid = field.id(obj)
+            if fid:
+                deleted.append(self.delete(fid))
         return 1
-    
-    def delete_indexes(self, obj):
-        pass
-    
+        
     def set(self, id, value, timeout = None):
+        timeout = timeout if timeout is not None else self.default_timeout
         value = self.pickler.dumps(value)
         return self._set(id,value,timeout)
     
@@ -217,8 +227,6 @@ and :ref:`ObjectNotFund <utility-exceptions>` exception.
         # if a subclass overrides it.
         return self.has_key(key)
 
-
-
     def delete_many(self, keys):
         """
         Set a bunch of values in the cache at once.  For certain backends
@@ -234,6 +242,9 @@ and :ref:`ObjectNotFund <utility-exceptions>` exception.
 
     # VIRTUAL METHODS
     
+    def keys(self, pattern = '*'):
+        raise NotImplementedError
+        
     def _set(self, id, value, timeout):
         raise NotImplementedError
     
@@ -245,22 +256,27 @@ and :ref:`ObjectNotFund <utility-exceptions>` exception.
             
     # DATASTRUCTURES
     
-    def list(self, *args, **kwargs):
-        '''Return an instance of :ref:`List <list-structure>`
-        for a given *id*.'''
-        raise NotImplementedError
+    def index_keys(self, id, timeout):
+        return Keys(id,timeout,self._keys)
     
-    def hash(self, *args, **kwargs):
-        '''Return an instance of :ref:`HashTable <hash-structure>`
-        for a given *id*.'''
-        raise NotImplementedError
+    def list(self, id, timeout = 0, **kwargs):
+        '''Return an instance of :class:`stdnet.List`
+for a given *id*.'''
+        return self.structure_module.List(self, id, timeout = timeout, **kwargs)
     
-    def unordered_set(self, *args, **kwargs):
-        '''Return an instance of :ref:`Set <set-structure>`'''
-        raise NotImplementedError
+    def hash(self, id, timeout = 0, **kwargs):
+        '''Return an instance of :class:`stdnet.HashTable` structure
+for a given *id*.'''
+        return self.structure_module.HashTable(self, id, timeout = timeout, **kwargs)
     
-    def ordered_set(self, *args, **kwargs):
-        '''Return an instance of :ref:`Ordered Set <orderedset-structure>`'''
-        raise NotImplementedError
+    def unordered_set(self, id, timeout = 0, **kwargs):
+        '''Return an instance of :class:`stdnet.Set` structure
+for a given *id*.'''
+        return self.structure_module.Set(self, id, timeout = timeout, **kwargs)
+    
+    def ordered_set(self, id, timeout = 0, **kwargs):
+        '''Return an instance of :class:`stdnet.OrderedSet` structure
+for a given *id*.'''
+        return self.structure_module.OrderedSet(self, id, timeout = timeout, **kwargs)
     
 
