@@ -1,22 +1,13 @@
 from itertools import izip
-from datetime import date
+from datetime import date, datetime
 from random import uniform
 
-from stdnet.stdtest import TestBase
-from stdnet import orm
-from stdnet.contrib.timeserie import models
+from stdnet.test import TestCase
 from stdnet.contrib.timeserie.utils import dategenerator, default_parse_interval
-from stdnet.utils import populate
-    
+from stdnet.utils import populate, todate
 
-class TimeSerie(models.TimeSerie):
-    data    = models.TimeSerieField(converter = models.DateConverter)
-    ticker  = orm.AtomField(unique = True)
-    
-    def __str__(self):
-        return '%s: %s' % (self.ticker,self.data.size())
+from models import TimeSeries, DateTimeSeries
 
-orm.register(TimeSerie)
 
 NUM_DATES = 300
 
@@ -25,13 +16,20 @@ values   = populate('float',NUM_DATES, start = 10, end = 400)
 alldata  = list(izip(dates,values))
 testdata = dict(alldata)
 
-class TestTimeSerie(TestBase):
+
+class TestTimeSeries(TestCase):
+    model = TimeSeries
+    mkdate = datetime
     
     def setUp(self):
-        TimeSerie(ticker = 'GOOG').save()
+        self.orm.register(self.model)
+        self.model(ticker = 'GOOG').save()
+        
+    def unregister(self):
+        self.orm.unregister(self.model)
         
     def get(self, ticker = 'GOOG'):
-        return TimeSerie.objects.get(ticker = ticker)
+        return self.model.objects.get(ticker = ticker)
         
     def filldata(self):
         d = self.get()
@@ -53,7 +51,7 @@ class TestTimeSerie(TestBase):
             self.assertEqual(y,target[1])
             for dt in dategenerator(x,y):
                 ts.data.add(dt,uniform(0,1))
-        ts.storestartend()
+        ts.save()
         self.assertEqual(ts.start,C)
         self.assertEqual(ts.end,D)
         
@@ -65,7 +63,7 @@ class TestTimeSerie(TestBase):
             if keyp:
                 self.assertTrue(key,keyp)
             keyp = key
-            data.pop(key)
+            data.pop(todate(key))
         self.assertEqual(len(data),0)
         
     def testitems(self):
@@ -76,34 +74,35 @@ class TestTimeSerie(TestBase):
             if keyp:
                 self.assertTrue(key,keyp)
             keyp = key
-            self.assertEqual(data.pop(key),value)
+            self.assertEqual(data.pop(todate(key)),value)
         self.assertEqual(len(data),0)
         
     def testInterval(self):
         '''Test interval handling'''
+        mkdate = self.mkdate
         ts = self.get()
         self.assertEqual(ts.start,None)
         self.assertEqual(ts.end,None)
         #
         #
-        A1   = date(2010,5,10)
-        B1   = date(2010,5,12)
+        A1   = mkdate(2010,5,10)
+        B1   = mkdate(2010,5,12)
         self.interval(A1,B1,[[A1,B1]],A1,B1)
         #
         #  original ->      A1      B1
         #  request  -> A2      B2
         #  interval -> A2  A1-
         #  range    -> A2          B1
-        A2   = date(2010,5,6)
-        B2   = date(2010,5,11)
+        A2   = mkdate(2010,5,6)
+        B2   = mkdate(2010,5,11)
         self.interval(A2,B2,[[A2,default_parse_interval(A1,-1)]],A2,B1)
         #
         #  original ->      A2      B1
         #  request  -> A3                B3
         #  interval -> A3  A2-       B1+ B3
         #  range    -> A3                B3
-        A3   = date(2010,5,4)
-        B3   = date(2010,5,14)
+        A3   = mkdate(2010,5,4)
+        B3   = mkdate(2010,5,14)
         self.interval(A3,B3,[[A3,default_parse_interval(A2,-1)],
                              [default_parse_interval(B1,1),B3]],A3,B3)
         #
@@ -117,38 +116,44 @@ class TestTimeSerie(TestBase):
         # request  -> A4  B4
         # interval -> A4      A3-
         # range    -> A4                         B3
-        A4   = date(2010,4,20)
-        B4   = date(2010,5,1)
+        A4   = mkdate(2010,4,20)
+        B4   = mkdate(2010,5,1)
         self.interval(A4,B4,[[A4,default_parse_interval(A3,-1)]],A4,B3)
         #
         # original -> A4                         B3
         # request  ->                A2                  B5
         # interval ->                             B3+    B5
         # range    -> A4                                 B5
-        B5   = date(2010,6,1)
+        B5   = mkdate(2010,6,1)
         self.interval(A2,B5,[[default_parse_interval(B3,1),B5]],A4,B5)
         #
         # original -> A4                                 B5
         # request  ->                                        A6    B6
         # interval ->                                     B5+      B6
         # range    -> A4                                           B6
-        A6   = date(2010,7,1)
-        B6   = date(2010,8,1)
+        A6   = mkdate(2010,7,1)
+        B6   = mkdate(2010,8,1)
         self.interval(A6,B6,[[default_parse_interval(B5,1),B6]],A4,B6)
         
     def testGetSet(self):
         ts = self.get()
-        dt = date(2010,7,1)
-        dt2 = date(2010,4,1)
+        mkdate = self.mkdate
+        dt = mkdate(2010,7,1)
+        dt2 = mkdate(2010,4,1)
         ts.data.add(dt,56)
         ts.data[dt2] = 78
         ts.save()
         self.assertEqual(ts.data.get(dt),56)
         self.assertEqual(ts.data[dt2],78)
         try:
-            ts.data[date(2010,3,1)]
+            ts.data[mkdate(2010,3,1)]
         except KeyError:
             pass
         else:
             self.fail('KeyError')
-        self.assertEqual(ts.data.get(date(2010,3,1)),None)
\ No newline at end of file
+        self.assertEqual(ts.data.get(mkdate(2010,3,1)),None)
+        
+
+class TestDateTimeSeries(TestTimeSeries):
+    model = DateTimeSeries
+    mkdate = date
\ No newline at end of file
