@@ -3,11 +3,19 @@
 
 from strings import *
 from manager import *
-from tables import *
+from finance import *
 from fktest import *
-from datefield import *
+from atomfields import *
+
+# Data-structure Fields
 from listfield import *
 from ordered_set import *
 from hashfield import *
+
+# Many to many Field
 from twitter import *
-from stdnet.contrib.timeserie.tests import *
\ No newline at end of file
+from testme import *
+
+# Contrib
+from stdnet.contrib.timeserie.tests import *
+from stdnet.contrib.djstdnet.tests import *
\ No newline at end of file
